
/***********************************************************************

TestDosGetDateTimeTzSetAndRuntime.c

Copyright (c) 2004-2023 Steven Levine and Associates, Inc.
All rights reserved.

This program is free software licensed under the terms of the GNU
General Public License, Version 2.  The GPL Software License can be found
in gnugpl2.txt or at http://www.gnu.org/licenses/licenses.html#GPL

2010-10-04 SHL Baseline
2010-10-04 SHL Extracted from watcom testme.c
2010-10-05 SHL Tweak messages to hopefully be more clear
2013-06-12 SHL Report timezone better
2023-02-14 SHL Report tzset() and time() better

Exit status	0	OK
		>0	Last API return code

For fast ow build with
  wcl386 TestDosGetDateTimeTzSetAndRuntime.c
For ow build with bldlevel and desc
  wmake -f makefile
For gcc build with
  gcc TestDosGetDateTimeTzSetAndRuntime.c
For vac build with
  icc TestDosGetDateTimeTzSetAndRuntime.c
    Will fail if INCLUDE configured for ow

***********************************************************************/

#include <stdlib.h>			// NULL _doserrno
#include <stdio.h>
#include <string.h>
#include <time.h>			// tzset...

#define INCL_DOSMISC			// TZ DosQuerySysInfo
#define INCL_DOSDATETIME		// TZ DosGetDateTime
#define INCL_DOSERRORS			// NO_ERROR
#include <os2.h>

#define	PGM_ID		"TestDosGetDateTimeTzSetAndRuntime"
#define VERSION		0
#define	REVISION	2

#ifndef __IBMC__
// Enable to find unreferenced variables
#pragma enable_message(14, 726);
#endif

int main(int argc, char **argv);

#ifdef __cplusplus
extern "C"
{
#endif

//=== main: Main program ===

int main(int argc,			// i: Argument count
	 char **argv)			// i: Argument list
{
  APIRET apiret = NO_ERROR;

  static char* dow[] = {
    "Sun",
    "Mon",
    "Tue",
    "Wed",
    "Thu",
    "Fri",
    "Sat"
  };
  DATETIME dt;			// Pointer to the DateTime data structure.
  PCSZ pTZ = getenv("TZ");
  time_t qsvTimeLowBeforeTzset;
  time_t timeLclSecBeforeTzset;	// from time()
  int timezoneBeforeTzset;
  int daylightBeforeTzset;
  time_t timeLclSec;		// from time
  struct tm tmLcl;
  struct tm tmGMT;
  time_t timeGMTSec;
  long lSec;
  PSZ psz;
  PSZ tznameBeforeTzset[2];
  CHAR szBuf[CCHMAXPATH];

  psz = strrchr(argv[0], '\\');
  if (!psz)
    psz = argv[0];
  else
    psz++;
  strcpy(szBuf, psz);
  psz = strrchr(szBuf, '.');
  if (psz)
    *psz = 0;
  strlwr(szBuf);
  printf("\n%s built on " __DATE__ " " __TIME__ " with ", szBuf);
#if defined(__WATCOMC__)
  // OpenWatcom 1.0 = 1200
  printf("__WATCOMC__ %d.%d (%d)\n", __WATCOMC__ / 1000, (__WATCOMC__ % 1000) / 100, __WATCOMC__);
#elif defined(__GNUC__)
  puts("GCC " __VERSION__ "");
#elif defined(__IBMC__)
  printf("VAC %s\n", __IBMC__ == 300 ? "3.08" : "3.65");
#else
  puts("*unknown*");
#endif

  printf("\nThe TZ variable is set to \"%s\"\n", pTZ ? pTZ : (PCSZ)"(null)");

  puts("\nBefore tzset()\n");

  apiret = DosQuerySysInfo(QSV_TIME_LOW, QSV_TIME_LOW,
			  &qsvTimeLowBeforeTzset, sizeof(qsvTimeLowBeforeTzset));
  if (apiret != NO_ERROR)
    printf("* DosQuerySysInfo failed with error %u\n", apiret);

  timeLclSecBeforeTzset = time(0);

  apiret = DosGetDateTime(&dt);
  if (apiret != NO_ERROR) {
    printf("* DosGetDateTime failed with error %u\n", apiret);
    return apiret;
  }

  printf("DosQuerySysInfo QSV_TIME_LOW says the seconds since 1/1/1970 00:00:00 is %u (%x)\n",
	 qsvTimeLowBeforeTzset, qsvTimeLowBeforeTzset);

  if (dt.timezone == -1) {
    printf("* DosGetDateTime reports that the kernel timezone has not been initialized\n");
    sprintf(szBuf, "is unset (%d)", dt.timezone);
  }
  else {
    sprintf(szBuf, "%d minutes (%d:%02d hours)",
	    dt.timezone, dt.timezone / 60, dt.timezone % 60);
  }

  printf("DosGetDateTime says the time is %02u:%02u:%02u:%02u and the date is %u/%u/%u\n"
	 "DosGetDateTime says the GMT offset is %s and the weekday is %s (%u)\n",
	 dt.hours, dt.minutes, dt.seconds, dt.hundredths,
	 dt.year, dt.month, dt.day,
	 szBuf,
	 dow[dt.weekday], dt.weekday);

  printf("\nThe runtime has initialized the tzname array to %s %s\n", tzname[0], tzname[1]);

# ifdef __IBMC__
# define timezone _timezone
# endif

  timezoneBeforeTzset = timezone;
  printf("The runtime has initialized the timezone variable %d:%02d:%02d hours\n",
	  timezone / (60 * 60), (timezone / 60) % 60, timezone % 60);
  
# ifdef __IBMC__
# undef timezone
# endif

  printf("The runtime has initialized the daylight flag to %d\n", daylight);
  daylightBeforeTzset = daylight;

  tznameBeforeTzset[0] = strdup(tzname[0]);
  tznameBeforeTzset[1] = strdup(tzname[1]);

  printf("\ntime() says the seconds since 1/1/1970 00:00:00 is %u (%x)\n",
	 timeLclSecBeforeTzset, timeLclSecBeforeTzset);

  lSec = timeLclSecBeforeTzset - qsvTimeLowBeforeTzset;
  if (lSec) {
    printf("* time() minus QSV_TIME_LOW is %d seconds (%d hours)\n",
	   lSec,
	   lSec / 3600);
  }

  puts("\nIssuing tzset()\n");
  tzset();

  if (strcmp(tznameBeforeTzset[0], tzname[0]) ||
      strcmp(tznameBeforeTzset[1], tzname[1])) {
    printf("* tzset has changed the tzname array to %s %s\n", tzname[0], tzname[1]);
  }

  timeLclSec = time(0);

  printf("time() says the seconds since 1/1/1970 00:00:00 is %u (%x)\n",
	 timeLclSec, 
	 timeLclSec);

  lSec = timeLclSec - timeLclSecBeforeTzset;
  if (lSec) {
    printf("* tzset has changed time() by %d seconds (%d hours)\n",
	   lSec,
	   lSec / 3600);
  }

  lSec = timeLclSec - qsvTimeLowBeforeTzset;
  if (lSec) {
    printf("* time() minus QSV_TIME_LOW is %d seconds (%d hours)\n",
	   lSec,
	   lSec / 3600);
  }

  tmLcl = *localtime(&timeLclSec);
  tmGMT = *gmtime(&timeLclSec);
  timeGMTSec = mktime(&tmGMT);


# ifdef __IBMC__
# define timezone _timezone
# endif
  if (timezone != timezoneBeforeTzset) {
    printf("* The runtime has changed the timezone variable to %d:%02d:%02d hours\n",
	   timezone / (60 * 60), (timezone / 60) % 60, timezone % 60);
  }
# ifdef __IBMC__
# undef timezone
# endif

  if (daylight != daylightBeforeTzset)
    printf("* The runtime has changed the daylight flag to %d\n", daylight);

  printf("The runtime says local time is %s", asctime(&tmLcl));
  printf("The runtime says local time is%s observing DST\n\n", tmLcl.tm_isdst ? "" : " not");

  printf("The runtime says GMT is %s", asctime(&tmGMT));
  printf("The runtime says GMT is%s observing DST\n\n", tmGMT.tm_isdst ? "" : " not");

  lSec = timeLclSec - timeGMTSec;	// negative if west of GMT
  printf("The GMT offset calculated using mktime results is %+05d\n",
	 (lSec / (60 * 60)) * 100 + (lSec / 60) % 60);

# ifndef __IBMC__
  lSec = - timezone;			// negative if west of GMT
# else
  lSec = - _timezone;			// negative if west of GMT
# endif
  if (tmLcl.tm_isdst)
    lSec += 60 * 60;
  printf("The GMT offset calculated using runtime timezone value and local isdst value %+05d\n",
	 (lSec / (60 * 60)) * 100 + (lSec / 60) % 60);

  if (dt.timezone != -1) {
    lSec = - dt.timezone;
    if (tmLcl.tm_isdst)
      lSec += 60;
    printf("The GMT offset calculated using DosGetDateTime timezone value and local isdst value is %+05d\n",
	   (lSec / 60) * 100 + lSec % 60);
  }
  else {
    // Check if runtime modified timezone
    apiret = DosGetDateTime(&dt);
    if (apiret != NO_ERROR) {
      printf("* DosGetDateTime failed with error %u\n", apiret);
      return apiret;
    }
    if (dt.timezone != -1)
      printf("* The runtime has changed the DosGetDateTime timezone from -1 to to %d\n", dt.timezone);
  }

  // printf("Exiting with rc %u\n", apiret);

  return apiret;

} // main

// The end
